(function( $ ) {
  var slice = Array.prototype.slice
  ,   Reactor = $.reactor =
    function( object ) {

    var eventListeners = {};

    if ( typeof object === 'undefined' ) {
      object = {};
    }



    object.removeAllListeners = function removeAllListeners() {
      eventListeners = {};
      return object;
    };



    object.removeListenersOn =
      function removeListenersOn( event ) {
        delete eventListeners[ event ];
        return object;
      };



    object.removeListener = function removeListener( event, listener ) {
      eventListeners[ event ] = _( eventListeners[event] ).without( listener );
      return object;
    };



    object.one = function one( event, callback ) {
      var removalCallback = function() {
        object.removeListener( event, callback );
        object.removeListener( event, removalCallback );
      };

      object.on( event, callback );
      object.on( event, removalCallback );

      return object;
    };


    /*
     * var listenerA = Listener.create()
     * ,   listenerB = Listener.create();
     * listenerB.notify( listenerA, 'oneEvent', 'anotherEvent' );
     */
    object.notify = function notify( otherListener, events ) {
      _(slice.call( arguments, 1 )).each(function( event ) {
        object.on( event, function() {
          otherListener.emit.apply( otherListener,
                                    [ event, object ]
                                    .concat( slice.apply( arguments )));
        });
      });
      return object;
    };


    /*
     * Listener.create()
     *   .addConvenience( 'eventOne', 'eventTwo' )
     *   .eventOne( function() {} ) //binds
     *   .eventTwo() //triggers
     */
    object.addConvenience = function addConvenience( names ) {
      _(slice.apply( arguments )).each( function( name ) {

        if ( typeof object[name] === 'undefined' ) {
          object[name] = function( optionalCallback ) {

            if ( arguments.length === 1 &&
                 typeof optionalCallback === 'function' ) {

              object.on( name, optionalCallback );

            } else {

              object.emit.apply( object,
                                 [name].concat( slice.apply( arguments )));

            }

            return object;
          };
        }
      });
      
      return object;
    };



    object.addListeners = function addListeners( convenience, listeners ) {
      if ( arguments.length === 1 ) {
        listeners = convenience;
        convenience = false;
      }

      _( listeners ).each( function( listener, name ) {
        object.addListener( name, convenience, listener );
      });

      return object;
    };



    object.on = object.addListener =
      function addListener( name, addConvenience, callback ) {
        if ( arguments.length === 2 ) {
          callback = addConvenience;
          addConvenience = false;
        }

        if ( name !== 'addListener' ) {
          object.emit( 'addListener', name, callback );
        }

        if ( typeof eventListeners[name] === 'undefined' ) {
          eventListeners[name] = [];
        }

        if ( addConvenience ) {
          object.addConvenience( name );
        }

        eventListeners[name].push( callback );

        return object;
      };


    
    object.emit = function( name, args ) {
      args = Array.prototype.slice.call( arguments, 1 );

      _(eventListeners[name]).each( function( callback ) {
        callback.apply( object, args );
      });

      return object;
    };


    
    return object;
  };
})( tiQuery );
