(function( tiQuery ) {
  var $        = tiQuery
  ,   root     = tiQuery.root
  ,   _        = $.globals._
  ,   console  = $.console

  $(root).addTemplate('perf')

  var testRow
  ,   templateItems
  ,   times = Ti.Platform.model === 'Simulator' ? 5000 : 500

  $(root).add({
    type: 'view',
    id: 'template_container',
    left: 10,
    top: 50,
    height: 49,
    width: 307,
  })

  $('#template_container').addTemplate('perf_test_template')

  $(root).add({
    type: 'label',
    id: 'log',
    text: '',
    left: 20,
    top: 130,
    height: 343,
    width: 286,
    font: {
      fontFamily: 'Helvetica',
      fontSize: 8
    }
  })

  $(root).add({
    type: 'label',
    id: 'queue',
    text: '0',
    top: 110,
    left: 275,
    height: 23,
    width: 40,
    font: {
      fontFamily: 'Helvetica',
      fontSize: 13
    }
  })

  var logView = $('#log')
  var queueView = $('#queue')
  var addLog = function( logString ) {
    var text = logView.first().text
    logView.first().text = text + logString + '\n'
    console.log(logString)
  }
  var clearLog = function() {
    logView.attr('text', '')
  }


  var workQueue = []

  var addWork = function( workFunction ) {
    if (!workQueue.length) {
      workQueue.push( workFunction )
      setTimeout( doWork, 500)
    } else {
      workQueue.push( workFunction )
    }
    queueView.attr('text', workQueue.length)
  }

  var doWork = function() {
    queueView.attr('text', workQueue.length)
    var workFunction = workQueue.shift()
    if (workFunction) {
      var result = workFunction()
      if (workQueue.length) {
        setTimeout( doWork, 500 )
      }
      queueView.attr('text', workQueue.length)
    }
  }

  var timeLog = function( name, times, workFunction ) {
    addWork(function() {
      var result = $.timeRepeat( name, times, workFunction )
      addLog( result.resultString )
    })
  }


  // Shared items for tests
  templateItems = $.loadJSON( 'perf_test_template' )
  testRow = $({
    type: 'tableRow',
    className: 'perf_test_row',
    id: 'perf_test_row',
    height: 55,
    selectionStyle: 'none'
  })

  var runAllTests = function() {
    console.log('runAllTests')
    clearLog()

    timeLog('createObject', times, function() {
      // Create a single Javascript object
      var foo = {
        width: 100,
        height: 100
      }
    })

    timeLog('createViewEmpty', times, function() {
      // Create a single view, no parameters
      var view = Ti.UI.createView()
    })

    timeLog('createView', times, function() {
      // Create a single view, setting width and ehight
      var view = Ti.UI.createView({
        width: 100,
        height: 100
      })
    })

    timeLog('createViewTiQuery', times, function() {
      // Create a single view using TiQuery
      var view = $({
        type: 'view',
        width: 100,
        height: 100
      })
    })

    timeLog('createRow', times, function() {
      // Create a table row
      var row = $({
        type: 'tableRow',
        className: 'perf_test_row',
        id: 'perf_test_row',
        height: 55,
        selectionStyle: 'none'
      })
    })


    timeLog('createAndAddRow', times, function() {
      // Create a table row and add the templated items
      var row = $({
        type: 'tableRow',
        className: 'perf_test_row',
        id: 'perf_test_row',
        height: 55,
        selectionStyle: 'none'
      })
      templateItems.forEach( function( item ) {
          row.add( item )
      })
    })

    timeLog('findTiQuery', times, function() {
      // Find an object of a class using TiQuery
      testRow.find('.date')
    })

    timeLog('findAndSetTiQuery', times, function() {
      // Find an object of a class using TiQuery and then set a parameter
      // on the object
      testRow.find('.date').attr('text', 'foobar')
    })

    timeLog('createAddSet', times, function() {
      // Create a row, add the template, and then set a value on the object
      var row = $({
        type: 'tableRow',
        className: 'perf_test_row',
        id: 'perf_test_row',
        height: 55,
        selectionStyle: 'none'
      })
      var items = $.loadJSON( 'perf_test_template' )
      items.forEach( function( item ) {
          row.add( item )
      })
      row.find('.date').text('foo').end()
    })

    timeLog('createSetAddTemplate', times, function() {
      // Create a row, set the value on the template, and then add it.
      var row = $({
        type: 'tableRow',
        className: 'perf_test_row',
        id: 'perf_test_row',
        height: 55,
        selectionStyle: 'none'
      })
      $.tizzle.find('.date', templateItems)[0].text = 'foo'
      templateItems.forEach( function( item ) {
          row.add( item )
      })
    })

    addWork(function() {
      addLog('Done running tests')
    })
    addLog('Done setting up tests')
  }

  $('#run_all').click(runAllTests)
})( tiQuery )